On 06/11/2015 10:07 AM, Ingo Molnar wrote:
The memory hotplug code uses sync_global_pgds() to synchronize updates
to the global (&init_mm) kernel PGD and the task PGDs. It does this
by iterating over the pgd_list - which list closely tracks task
creation/destruction via fork/clone.

But we want to remove this list, so that it does not have to be
maintained from fork()/exit(), so convert the memory hotplug code
to use the task list to iterate over all pgds in the system.

Also improve the comments a bit, to make this function easier
to understand.

Only lightly tested, as I don't have a memory hotplug setup.

Cc: Andrew Morton<a...@linux-foundation.org>
Cc: Andy Lutomirski<l...@amacapital.net>
Cc: Borislav Petkov<b...@alien8.de>
Cc: Brian Gerst<brge...@gmail.com>
Cc: Denys Vlasenko<dvlas...@redhat.com>
Cc: H. Peter Anvin<h...@zytor.com>
Cc: Linus Torvalds<torva...@linux-foundation.org>
Cc: Peter Zijlstra<pet...@infradead.org>
Cc: Thomas Gleixner<t...@linutronix.de>
Cc: Waiman Long<waiman.l...@hp.com>
Signed-off-by: Ingo Molnar<mi...@kernel.org>
---
  arch/x86/mm/init_64.c | 30 +++++++++++++++++-------------
  1 file changed, 17 insertions(+), 13 deletions(-)

diff --git a/arch/x86/mm/init_64.c b/arch/x86/mm/init_64.c
index 3fba623e3ba5..1921acbd49fd 100644
--- a/arch/x86/mm/init_64.c
+++ b/arch/x86/mm/init_64.c
@@ -160,8 +160,8 @@ static int __init nonx32_setup(char *str)
  __setup("noexec32=", nonx32_setup);

  /*
  * When memory was added/removed make sure all the processes MM have
  * suitable PGD entries in the local PGD level page.
   */
  void sync_global_pgds(unsigned long start, unsigned long end, int removed)
  {
@@ -169,29 +169,33 @@ void sync_global_pgds(unsigned long start, unsigned long 
end, int removed)

        for (address = start; address<= end; address += PGDIR_SIZE) {
                const pgd_t *pgd_ref = pgd_offset_k(address);
-               struct page *page;
+               struct task_struct *g, *p;

                /*
-                * When it is called after memory hot remove, pgd_none()
-                * returns true. In this case (removed == 1), we must clear
-                * the PGD entries in the local PGD level page.
+                * When this function is called after memory hot remove,
+                * pgd_none() already returns true, but only the reference
+                * kernel PGD has been cleared, not the process PGDs.
+                *
+                * So clear the affected entries in every process PGD as well:
                 */
                if (pgd_none(*pgd_ref)&&  !removed)
                        continue;

                spin_lock(&pgd_lock);
-               list_for_each_entry(page,&pgd_list, lru) {
-                       pgd_t *pgd;
+
+               for_each_process_thread(g, p) {
+                       pgd_t *pgd = p->mm->pgd;
                        spinlock_t *pgt_lock;

-                       pgd = (pgd_t *)page_address(page) + pgd_index(address);
-                       /* the pgt_lock only for Xen */
-                       pgt_lock =&pgd_page_get_mm(page)->page_table_lock;
+                       if (!p->mm)
+                               continue;
@@ -160,8 +160,8 @@ static int __init nonx32_setup(char *str)

  /*
- * When memory was added/removed make sure all the processes MM have
- * suitable PGD entries in the local PGD level page.
+ * When memory was added/removed make sure all the process MMs have
+ * matching PGD entries in the local PGD level page as well.
   */
  void sync_global_pgds(unsigned long start, unsigned long end, int removed)

pgd was initialized to p->mm->pgd before the "p->mm" check is done. Shouldn't the initialization be moved after that.

Cheers,
Longman
--
To unsubscribe from this list: send the line "unsubscribe linux-kernel" in
the body of a message to majord...@vger.kernel.org
More majordomo info at  http://vger.kernel.org/majordomo-info.html
Please read the FAQ at  http://www.tux.org/lkml/

Reply via email to